package bsoc.alpha;

import EDU.gatech.cc.is.util.*;
import EDU.gatech.cc.is.abstractrobot.*;
import java.util.*;
import bsoc.nn.*;  
import bsoc.division.*;  

/**
* A player controlled by an alpha-net. 
*/

public class AlphaPlayer extends ControlSystemSS implements Player {

	private Net net;
	private static AlphaConnector conn = new AlphaConnector (-1); 
	private static RandomWgt rw = new RandomWgt (); 

	/**
	* Generates an alpha-player where the parameters of the net (the weights)
	* are random numbers.
	*/

	public AlphaPlayer () {
		super ();
		net = new Net (conn);
		net.setWeightsRandom (rw);
	}

	/**
	* Controls the movements of an AlphaPlayer.<p>
	* This is done by mapping the sensor- and actor- values of the 
	* abstract robot to the neuronal net and by calculating the neuronal
	* net.
	*/

	public int takeStep() {
		int i, j;
		short maxValue, val;
		int maxIndex;

		Octant opponentsOctant = new Octant ();
		Octant teammatesOctant = new Octant ();
		Octant opponentsGoalOctant = new Octant ();
		Octant ourGoalOctant = new Octant ();
		Octant ballOctant = new Octant ();

		Layer in = net.getInputLayer ();
		Layer out = net.getOutputLayer ();

		long curr_time = abstract_robot.getTime();
		 
		opponentsOctant.addVec2Array (abstract_robot.getOpponents(curr_time), 0.2);
		teammatesOctant.addVec2Array (abstract_robot.getTeammates(curr_time), 0.2);
		opponentsGoalOctant.addVec2 (abstract_robot.getOpponentsGoal(curr_time), 0.5);
		ourGoalOctant.addVec2 (abstract_robot.getOurGoal(curr_time), 1);
		ballOctant.addVec2 (abstract_robot.getBall(curr_time), 1);

		j = 0;
		for (i=0; i<8; i++) {
			in.setValueAt (j, opponentsOctant.getValueAt (i));
			j++;  
		}
		for (i=0; i<8; i++) {
			in.setValueAt (j, teammatesOctant.getValueAt (i));
			j++;  
		}
		for (i=0; i<8; i++) {
			in.setValueAt (j, opponentsGoalOctant.getValueAt (i));
			j++;  
		}
		for (i=0; i<8; i++) {
			in.setValueAt (j, ourGoalOctant.getValueAt (i));
			j++;  
		}
		for (i=0; i<8; i++) {
			in.setValueAt (j, ballOctant.getValueAt (i));
			j++;  
		}

		net.recalculate ();

		maxValue = out.getValueAt (0);
		maxIndex = 0;
		for (i=1; i<8; i++) {
			val = out.getValueAt (i);
			if (val > maxValue) {
				maxValue = val;
				maxIndex = i;
			}
		} 	

		abstract_robot.setSteerHeading (curr_time, octantToRad (maxIndex));
		abstract_robot.setSpeed (curr_time, valueToSpeed (maxValue));	
		
		return(CSSTAT_OK);
	}

	private double octantToRad (int oct) {
		switch (oct) {
			case 0: return 0.0000;
			case 1: return 0.7853;
			case 2: return 1.5707;
			case 3: return 2.3561;
			case 4: return 3.1415;
			case 5: return 3.9269;
			case 6: return 4.7123;
			case 7: return 5.4977;
			default: return 0.0000;
		}
	}			 
	
	private double valueToSpeed (int val) {
		return ((double)val / (double)Params.maxValue);
	}

	public String toString () {
		return "<alpha>";
	}
}

class Octant extends Object {

	private final static double MAX_LOOK_DISTANCE = 4.0;
	private double [] vals = new double [8];

	Octant () {
		super ();
	}

	public void addVec2 (Vec2 v, double weight) {
		if (v.r < MAX_LOOK_DISTANCE) 
			vals [v.octant()] += (1 - (v.r / MAX_LOOK_DISTANCE)) * weight;
	}

	public void addVec2Array (Vec2[] vArray, double weight) {
		int i;
		for (i=0; i<vArray.length; i++) {
			addVec2 (vArray[i], weight);
		}
	}

	public short getValueAt (int i) {
		return (short)(vals [i] * Params.maxValue);
	} 			
}	

