package bsoc.division;

import java.util.*;
import java.io.*;
import bsoc.util.*;

/**
* Division manages a set of teams and the handling of a round of matches.
* It holds the results of matches.
*/
public abstract class Division extends Thread implements Serializable {

	public static int CHANGED = 0;

	DTable dTable = new DTable ();
	IComb iComb;
	boolean keepRunning = true;
	int playedMatches = 0;
	Vector listeners = new Vector();

	public Division () {
		super();
	}

	/**
	* Creates a new Division from a Vector of Teams.
	*/
	public Division (Vector teams) {
		this();	
		setTeams(teams);
	}


	public void setTeams (Vector teams) {
		Enumeration teamsEnum;
		int i = 0;

		iComb = new IComb (teams.size());
		teamsEnum = teams.elements();
		while (teamsEnum.hasMoreElements()) {
			addTeam ((Team)teamsEnum.nextElement(), i);
			i++;
		}
	}


	/**
	* Plays matches. The selection
	* of teams follows the implementation of IComb.
	* @see bsoc.util.IComb
	*/
	public void run () {
		TeamPair tp;

		while (keepRunning) {
			performEvent (CHANGED);
			tp = nextTeamPair ();
			playOneMatch (tp.getA (), tp.getB ());
			playedMatches++;
			performEvent (CHANGED);
			if (shouldStop()) stopDivision();
		}
		performEvent (CHANGED);
	}

	public boolean shouldStop () {
		return false;
	}

	void playOneMatch (Team ta, Team tb) {
		MatchResult mr;

		mr = play (ta, tb);
		addMatchResult (mr);
	}

	/**
	* Holds the implementation of the playing of one game.
	*/
	public abstract MatchResult play (Team ta, Team tb);

	TeamPair nextTeamPair () {
		IPair ip;
		Team a, b;
		ip = iComb.next ();
		a = teamForTeamId (ip.getA ());
		b = teamForTeamId (ip.getB ());
		return (new TeamPair (a, b));
	}			
	
	public Team teamForTeamId (int id) {
		return dTable.teamForTeamId (id);
	}

	public int teamIdForTeam (Team t) {
		return dTable.teamIdForTeam (t);
	}

	public DTableEntry entryForTeamId (int id) {
		return dTable.entryForTeamId (id);
	}

	public DTableEntry entryForTeam (Team t) {
		return dTable.entryForTeam (t);
	}

	void addTeam (Team t, int id) {
		dTable.addEntry (new DTableEntry (t, id));
	}

	void addMatchResult (MatchResult mr) {
		int nrTeamA, nrTeamB, goalsTeamA, goalsTeamB;
		Team teamA, teamB;

		teamA = mr.getTeamA ();
		teamB = mr.getTeamB ();

		goalsTeamA = mr.getGoalsA ();
		goalsTeamB = mr.getGoalsB ();

		dTable.addMatchResult (teamA, teamB, goalsTeamA, goalsTeamB);
	}

	public void stopDivision() {
		keepRunning = false;
	}

	public String toInfoString () {
		String str;
		int i;
		Team t;

		str = "---Division\n";
		str += "playedMatches = " + playedMatches + "\n";
		str += "keepRunning = " + keepRunning + "\n";
		str += "listeners(count) = " + listeners.size() + "\n";
		str += "---Division Table by Points\n";
		dTable.sort();
		str += dTable.toInfoString ();	 
		return str;
	}

	public Enumeration dTableEntries () {
		return dTable.entries ();
	}

	public boolean equals (Object o) {
		if (! (o instanceof Division)) return false;
		else {Division d = (Division)o;
			if (
				(dTable.equals(d.dTable)) &&
				(iComb.equals(d.iComb))) return true;
			else return false;
		}
	}

	public int getPlayedMatches () {
		return playedMatches;
	}

        public void performEvent (int id) {
                DivisionListener l;
		Enumeration enum;

                enum = listeners.elements();
                while (enum.hasMoreElements()) {
                        l = (DivisionListener)enum.nextElement();
                        l.divisionEventPerformed (id);
                }
        }

        public void addListener (DivisionListener listener) {
                listeners.addElement (listener);
        }

        public void removeListener (DivisionListener listener) {
                listeners.removeElement (listener);
        }

} 

