package bsoc.nn;

import bsoc.genetic.*;
import java.util.*;
import java.io.*;

/**
 * Is an artificiel neural net.
 * <P>
 * A net contains out of an ordered list of layers. The first layer is called the input-layer and the
 * last layer is the output layer. The layers are lists of layer-nodes. A layer-node is a
 * neuron if it is not in the input layer. Connections between neurons and layer-nodes can be defined 
 * via synapses. The structure (its connections) of the net is defined by the net-connector that must be
 * used as an argumet to the constructor. Diverse parameters for synapses and neurons can also be defined
 * with the net-Connector.
 * <P>
 * see@ Neuron
 * see@ Layer
 */

public class Net implements Serializable, ToBeConnectedInterface, Crossable {
	
	Vector ls = new Vector ();
	NetConnector netConnector;
	


	public Net (NetConnector nc) {
		netConnector = nc;		
		nc.initLayers (this);
		nc.connectNet (this);
	}

	public NeuronLayer getOutputLayer () {
		return (NeuronLayer)(layerAt (layerCount () - 1));
	}

	public Layer getInputLayer () {
		return (layerAt (0));
	}

	/**
	* Creates a child net. The structure is taken from this net. The weights are set by
	* n-point crossover. Mutation is controlled by a mutator.
	*/

	public Crossable newChild (Crossable otherParent, CrossoverSwitch cs, Mutator mut) {
		Net otherNet, childNet;

		otherNet = (Net)otherParent;
		childNet = new Net (netConnector);
		childNet.setWeightsCrossover (this, otherNet, cs, mut);

		return childNet;
	}

	/**
	* Sets the weights of a child net applying crossover and mutation to parent nets.
	*/
	void setWeightsCrossover (Net netA, Net netB, CrossoverSwitch cs, Mutator mut) {
		Enumeration enumA, enumB, enumChild;
		Synapse synA, synB, synChild;
		RandomWgt rw = new RandomWgt ();

		enumA = netA.synapses();								
		enumB = netB.synapses();								
		enumChild = this.synapses();
	
		while (enumA.hasMoreElements()) {
			synA = (Synapse)enumA.nextElement();
			synB = (Synapse)enumB.nextElement();
			synChild = (Synapse)enumChild.nextElement();

  			if (mut.isMutation()) synChild.setWeightRandom(rw);
			else if (cs.takeA()) synChild.setWeight(synA.getWeight());
			else synChild.setWeight(synB.getWeight());
		}
			
	}		

	public void setInputLayerValuesRandom (RandomValue rv) {
		Layer il;
		Enumeration e;

		il = getInputLayer ();
		il.setValuesRandom (rv);
	}

	public void setWeightsRandom (RandomWgt rw) {
		Synapse syn;
		Enumeration e;

		for (e = synapses (); e.hasMoreElements ();) {
			syn = (Synapse)e.nextElement (); 
			syn.setWeightRandom (rw);
		}
	}

	public void setWeightsRandom () {
		RandomWgt rw = new RandomWgt();
		setWeightsRandom (rw);
	}

	/**
	* Sets the weights of the net according to the weights of the parent nets by means 
	* of an n-point crossover algorithm.
	**/

	public void setWeightsCrossover (Net father, Net mother, Random r) {
		Synapse syn, synf, synm;
		Enumeration e, ef, em;
		int count, offset, interval;
		boolean fromFather;
	
		count = 0;
		interval = 20;
		if (Math.abs(r.nextInt () % 1000) > 500) fromFather = true;
		else fromFather = false;
		offset = Math.abs (r.nextInt () % 20);
 		e = synapses ();
		ef = father.synapses ();
		em = mother.synapses ();
		while (e.hasMoreElements ()) {
			syn = (Synapse)e.nextElement ();
			synf = (Synapse)ef.nextElement ();
			synm = (Synapse)em.nextElement ();
			if (fromFather) {
				syn.setWeight (synf.getWeight ());
				if ((count + offset) % interval == 0) fromFather = false;
			}
			else {		
				syn.setWeight (synm.getWeight ());
				if ((count + offset) % interval == 0) fromFather = true;
			}
			count ++;
 		}
	}



	public void addLayer (Layer l) {
		ls.addElement (l);
	}		

	public Layer layerAt (int i) {
		return (Layer) ls.elementAt (i);
	}

	public int layerCount () {
		return ls.size ();
	}


	public String toString () {
		int i, size;
                String str = new String ();
		
		size = this.layerCount ();
		str += "--- Net BEGIN ---\n";
		for (i=0; i < size; i++) {
			Layer l;
			l = this.layerAt (i);
			str += l.toString () + "\n";
		}
		str += "--- Net END ---\n";
                return str;
	}


	Vector compareWeights (Net net1) {
		Synapse syn, syn1;
		Enumeration e, e1;
		short w, w1;
                Vector result = new Vector ();

		e = synapses ();
		e1 = net1.synapses ();
 		while (e.hasMoreElements ()) {
			syn = (Synapse)e.nextElement (); 
			syn1 = (Synapse)e1.nextElement (); 
			w = syn.getWeight ();
			w1 = syn1.getWeight ();
			if (w == w1) result.addElement (new Integer(0));
			else result.addElement (new Integer(1));
		}
		return result;
	}

	/**
	* Writes a connection statistic to an output stream.
	* This stream can be used to visualize the structure of the net.
	* <P>
	* The output stream is a table who's colums are delimited by semicolums.
	* The first line contains columheaders.
	**/
	public void writeConnStatToStream (PrintWriter str) {
		int i, size;
		Layer l;

		str.println ("LFr;NeuFr;LTo;NeuTo;conn");
		size = layerCount ();
		for (i=0; i<size; i++) {
			l = layerAt (i);
			csLayer (i, l, str);
		}
	}

	void writeSynStatToStream (PrintWriter str) {
		Enumeration enl;
		NeuronLayer nl;
		Neuron neu;
		int i, j, size, sc;

		enl = neuronLayers ();
		j = 1;
		str.println ("ni ;li ; synCount");
		while (enl.hasMoreElements ()) {
			nl = (NeuronLayer)enl.nextElement ();
			size = nl.size ();
			for (i=0; i<size; i++) {
				neu = (Neuron)nl.layerNodeAt (i);
				sc = neu.synapseCount ();
				str.println (i+" ;"+j+" ;"+sc);
			}
			j++;
		}
	} 
 
	void csLayer (int li, Layer l, PrintWriter str) {
		int i, size;
		LayerNode ln;

		size = l.size ();
		for (i=0; i<size; i++) {
			ln = l.layerNodeAt (i);
			csLayerNode (li, l, i, ln, str);
		}
	}

	void csLayerNode (int lfi, Layer lf, int ni, LayerNode ln, PrintWriter str) {
		int i, size;
		Layer l;

		size = layerCount ();
		for (i=0; i<size; i++) {
			l = layerAt (i);
			csLayerNodeToLayer (lfi, ni, ln, i, l, str);
		}
	}

	void csLayerNodeToLayer (int lfi, int lnfi, LayerNode lnf, int lti, Layer lt, PrintWriter str) {
		int i, size;
		LayerNode ln;

		size = lt.size ();
		for (i=0; i<size; i++) {
			ln = lt.layerNodeAt (i);
			csLayerNodeToLayerNode (lfi, lnfi, lnf, lti, i, ln, str);
		}
	}

	void csLayerNodeToLayerNode (int lfi, int lnfi, LayerNode lnf, int lti, int lnti, LayerNode lnt, PrintWriter str) {
		if (lnf.isConnectedToLayerNode (lnt)) {
			str.print (lfi+";"+lnfi+";"+lti+";"+lnti+";");	
			str.println ("1");
		}
		else {
//			str.print (lfi+";"+lnfi+";"+lti+";"+lnti+";");	
//			str.println ("0");
		}		

	}

	void calculate () {
		Layer ol;

		ol = getOutputLayer ();
		ol.calculate ();
	}

	void resetCalculated () {
		int i, size;
		Layer l;

		size = layerCount ();
		for (i=1; i<size; i++) {
			l = layerAt (i);
			l.resetCalculated ();
		}
	}	

	public void recalculate () {
		resetCalculated ();
		calculate ();
	} 

	/**
	* Compares the values of the layer-nodes (or neurons) of two nets.
	* Can be used for testing
	**/
        public boolean equalsInValues (Object o) {
                Layer l, thisL;
                Enumeration ls, thisLs;
                boolean equals = true;
                Net net;
 
                if (! (o instanceof Net)) return false;
                else {
                        net = (Net)o;
                        ls = net.layers ();
                        thisLs = layers ();
                        while (ls.hasMoreElements () && thisLs.hasMoreElements () && equals) {
                                l = (Layer)ls.nextElement ();
                                thisL = (Layer)thisLs.nextElement ();
                                if (! l.equalsInValues (thisL)) equals = false;
                        }
                        if (equals == true) {
                                if (thisLs.hasMoreElements () || ls.hasMoreElements ()) return false;
                                else return true;
                        }
                        else return false;
                }
        }

	/**
	* Compares the weights of the synapses of two nets.
	* Can be used for testing
	**/
        public boolean equalsInWeights (Object o) {
                NeuronLayer l, thisL;
                Enumeration ls, thisLs;
                boolean equals = true;
                Net net;
 
                if (! (o instanceof Net)) return false;
                else {
                        net = (Net)o;
                        ls = net.neuronLayers ();
                        thisLs = neuronLayers ();
                        while (ls.hasMoreElements () && thisLs.hasMoreElements () && equals) {
                                l = (NeuronLayer)ls.nextElement ();
                                thisL = (NeuronLayer)thisLs.nextElement ();
                                if (! l.equalsInWeights (thisL)) equals = false;
                        }
                        if (equals == true) {
                                if (thisLs.hasMoreElements () || ls.hasMoreElements ()) return false;
				else return true;
			}
			else return false;
		}
	}

	/**
	* Compares the weights and values of two nets.
	* Can be used for testing
	**/
	public boolean equals (Object o) {
		return equalsInValues (o) && equalsInWeights (o);
	}

	Enumeration synapses () {
		return new EnumSynapsesOfNet (this);
	}	

	/**
	* Iterates through the layers of the net. Returns layers.
	**/
	public Enumeration layers () {
		return new EnumLayersOfNet (this);
	}

	/**
	* Iterates through the layers containing neurons. This are all layers except the input layer. 
	* Returns Layers.
	**/
	public Enumeration neuronLayers () {
		return new EnumNeuronLayersOfNet (this); 
	}	 
}

class EnumSynapsesOfNet implements Enumeration {

	Enumeration enl;
	int i, size;
	Vector ls;
	Object nextSyn = null;

	EnumSynapsesOfNet (Net net) {
		NeuronLayer nl;

		size = net.layerCount ();
		if (size >= 2) {
			i = 1;
			ls = net.ls;
			nl = (NeuronLayer)ls.elementAt (i);
			i++;
			enl = nl.synapses ();	
			nextSyn = nextSyn ();
		}	
	}
	
	Object nextSyn () {
		NeuronLayer nl;
		if ( enl.hasMoreElements ()) return enl.nextElement ();
		else if (i >= size) return null;
		else {
			nl = (NeuronLayer)ls.elementAt (i);
			i++;
			enl = nl.synapses ();		
			return nextSyn ();
		}
	}

	public boolean hasMoreElements () { 
		if (nextSyn == null) return false;
		else return true;
	} 

	public Object nextElement () {
		Object o;

		o = nextSyn;
		nextSyn = nextSyn ();
		return o;
	}
}

class EnumLayersOfNet implements Enumeration {

	int i, size;
	Vector ls;

	EnumLayersOfNet (Net net) {
		NeuronLayer nl;

		size = net.layerCount ();
		i = 0;
		ls = net.ls;
	}
	
	public boolean hasMoreElements () {
		if (i >= size) return false;
		else return true;
	}

	public Object nextElement () {
		Object o;

		o = ls.elementAt (i);
		i++;
		return o;
	}

}
class EnumNeuronLayersOfNet implements Enumeration { 
 
	int i, size; 
	Vector ls; 
 
	EnumNeuronLayersOfNet (Net net) { 
		NeuronLayer nl; 
 
		size = net.layerCount (); 
		i = 1; 
		ls = net.ls; 
	} 
	 
	public boolean hasMoreElements () { 
		if (i >= size) return false; 
		else return true; 
	} 
 
	public Object nextElement () { 
		Object o; 
 
		o = ls.elementAt (i); 
		i++; 
		return o; 
	} 

} 

