package bsoc.nn;

import java.util.*;
import bsoc.util.*;
import java.io.*;

/**
* The netconnector allowes to define the structure of neuronal nets and
* parameters for synapses and neurons.
**/


public class NetConnector extends Object implements Serializable {

	IntVector nodesPerLayer;
	IntVector connProbMatrix;
	int seed;
	Random ran;
	Vector connMatrix = new Vector ();
        Weighter wgt;
        Transfer trans;

	/**
	* Generates a default net. Four layers with four neurons each. Mainly for test purposes.
	**/
	public NetConnector () {
                nodesPerLayer = defaultNodesPerLayer ();
                connProbMatrix = defaultConnProbMatrix ();
                wgt = new Weighter ();
                this.trans = new Transfer ();
		initConnMatrix ();
	}

	/**
	* Allows to construct a net-connector with parameters to build a net.
	*
	* @param nodesPerLayer	A list of integer values spezifying the number of layers 
	*			and the number of neurons per layer. 
	*			The first entry spezifies the input layer.
	* @param connProbMatrix	A list of lists of integer values spezifying the 
	*			probabillity of a connection of the neurons of a layer 
	*			to be connected to another layer. Values must be in percent.
	* @param wgt		The weighter used to preprocess input values to synapses.
	* @param trans		The transfer function used in neurons.
	*/

	public NetConnector (int seed, IntVector nodesPerLayer, IntVector connProbMatrix, Weighter wgt, Transfer trans) {
                this.nodesPerLayer = nodesPerLayer;
                this.connProbMatrix = connProbMatrix;
                this.wgt = wgt;
                this.trans = trans;
		initConnMatrix ();
	}

        IntVector defaultNodesPerLayer () {
                IntVector npl = new IntVector ();
                npl.addElement (4);
                npl.addElement (4);
                npl.addElement (4);
                return npl;
        }

        IntVector defaultConnProbMatrix () {
                IntVector cpm = new IntVector ();
                IntVector cpl;

                        cpl = new IntVector ();
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (100);
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (0);
                        cpl.addElement (100);
                cpm.addElement (cpl);
                        cpl = new IntVector ();
                        cpl.addElement (0);
                        cpl.addElement (0);
                        cpl.addElement (100);
                cpm.addElement (cpl);

                return cpm;
        }


        Neuron getNewNeuron () {
                return new Neuron (trans);
        }

        Synapse getNewSynapse () {
                return new Synapse (wgt);
        }

	void initConnMatrix () {
		int lfromi;
		Vector lfrom;
                int lCount = nodesPerLayer.size ();
		
		ran = new Random (seed);
		connMatrix.insertElementAt (new Vector (), 0);
		for (lfromi=1; lfromi<lCount; lfromi++) {
			lfrom = new Vector ();
			initCMLayerFrom (lfrom, lfromi);
			connMatrix.insertElementAt (lfrom, lfromi);
		} 
	}

	void initCMLayerFrom (Vector lfrom, int lfromi) {
		int nfromi;
		Vector nfrom;

		for (nfromi=0; nfromi<nodesPerLayer.intAt(lfromi); nfromi++) {
			nfrom = new Vector ();
			initCMNeuronFrom (nfrom, nfromi, lfromi);
			lfrom.insertElementAt (nfrom, nfromi);
		}
	}	
		
	void initCMNeuronFrom (Vector nfrom, int nfromi, int lfromi) {
		int ltoi;
		Vector lto;

		for (ltoi=0; ltoi<lfromi; ltoi++) {
			lto = new Vector ();
			initCMLayerTo (lto, ltoi, nfromi, lfromi);
			nfrom.insertElementAt (lto, ltoi);
		}
	}

	void initCMLayerTo (Vector lto, int ltoi, int nfromi, int lfromi) {
		int ntoi;

		for (ntoi=0; ntoi<nodesPerLayer.intAt(ltoi); ntoi++) {
			if (hasConnection (ltoi, lfromi)) {
				lto.addElement (new Integer (ntoi));
			}
		}		
	}

	boolean hasConnection (int ltoi, int lfromi) {
		if (ran.nextDouble () <= prob (ltoi, lfromi)) return true;
		else return false;
	}

	double prob (int ltoi, int lfromi) {
		double prob;
                IntVector connProbIntVector;
//		System.out.println ("++ prob ltoi="+ltoi+" lfromi="+lfromi);
                connProbIntVector = connProbMatrix.intVectorAt (lfromi);
		prob = ((double)connProbIntVector.intAt(ltoi)) / 100.0;	
//		System.out.println ("prob="+prob);
		return prob;	
	}	

	public void initLayers (ToBeConnectedInterface net) {
		int i, j, lCount;
                Layer l;
                NeuronLayer nl;
 
                l = new Layer (nodesPerLayer.intAt (0));
		net.addLayer (l);
                lCount = nodesPerLayer.size ();
		for (i=1; i<lCount; i++) {
                        nl = new NeuronLayer ();
                        for (j=0; j<nodesPerLayer.intAt(i); j++) {
                                nl.addNeuron (getNewNeuron ());
                        }
			net.addLayer (nl);
		}
	}

	public void connectNet (ToBeConnectedInterface net) {
		int lfromi, size;
		
		size = net.layerCount ();
		for (lfromi=1; lfromi<size; lfromi++) {
			connectLayer (net, lfromi);
		}
	}

	void connectLayer (ToBeConnectedInterface net, int lfromi) {
		Layer lfrom;
		int nfromi, size;

		lfrom = net.layerAt (lfromi);
		size = lfrom.size ();
		for (nfromi=0; nfromi<size; nfromi++) {
			connectNeuron (net, lfromi, nfromi, lfrom);
		}
	}

	void connectNeuron (ToBeConnectedInterface net, int lfromi, int nfromi, Layer lfrom) {
		Neuron nfrom;
		int ltoi;

		nfrom = (Neuron)lfrom.layerNodeAt (nfromi);
		for (ltoi=0; ltoi<lfromi; ltoi++) {
			connectNeuronToLayer (net, lfromi, nfromi, nfrom, ltoi);
		}
	}

	void connectNeuronToLayer (ToBeConnectedInterface net, int lfromi, int nfromi, Neuron nfrom, int ltoi) {
		Layer lto;
		Vector vlfrom, vnfrom, vlto;
		int i, size;
		Integer indexTo;
                Synapse syn;
		
		lto = net.layerAt (ltoi);
		vlfrom = (Vector)connMatrix.elementAt (lfromi);
		vnfrom = (Vector)vlfrom.elementAt (nfromi);
		vlto = (Vector)vnfrom.elementAt (ltoi);
		size = vlto.size ();
		for (i=0; i<size; i++) {
			indexTo = (Integer)vlto.elementAt (i);
                        syn = getNewSynapse ();
                        syn.connectLayerNode (lto.layerNodeAt (indexTo.intValue ()));
			nfrom.addSynapse (syn);
		}
	}
}

