package bsoc.nn;

import java.util.*;
import java.io.*;

/**
* The weighter is used to preprocess actiavtion values of preceeding neurons. Weighting
* is a multiplication of the incoming value with a value between min- and max factor. What
* factor is taken depends on the weight of the synapse. The weight of a synapse is a value
* out of the intervall [0 .. maxw].
* <p>
* <code>
* <dd>f = maxf + ((maxf - minf) * (w / maxw))
* <dd>f ... Factor
* <dd>maxf ... maxFactor
* <dd>minf ... minFactor
* <dd>w ... weight
* <dd>maxw ... maxWeight
* <code/>
* <p>The weighter is implemented as an array of values where result values can be calculated
* by accessing these values. This should speed up the calculation of neuronal nets.
**/


public class Weighter implements Serializable {
	
	Vector vals = new Vector ();

	public Weighter () {
                initVals (Params.maxValue, Params.maxWgt, (float)0.1, (float)1.5);
	}

	public Weighter (float minFactor, float maxFactor) {
                initVals (Params.maxValue, Params.maxWgt, minFactor, maxFactor);
	}

	public Weighter (double minFactor, double maxFactor) {
                initVals (Params.maxValue, Params.maxWgt, (float)minFactor, (float)maxFactor);
	}

	void initVals (short maxValue, short maxWeight, float minFactor, float maxFactor) {
		short val, wgt;
		float step;
		Vector wgts;

		step = (maxFactor - minFactor) / maxWeight;		
		for (val=0; val<maxValue; val++) {
			wgts = new Vector ();
			for (wgt=0; wgt<maxWeight; wgt++) {
				wgts.addElement (result(val, wgt, minFactor, step));
			}
			vals.addElement (wgts);
		} 		
	}

	Integer result (short val, short wgt, float minFactor, float step) {
		return new Integer ((int)(val * (minFactor + wgt * step)));
	}

	int getWeightedValue (short val, short wgt) {
		if (val > 0) {
			if (wgt > 0) {
				return getWeightedValueForPositives (val, wgt);
			}
			else {
				return - getWeightedValueForPositives (val, (short)(-wgt));
			}
		}
		else {
			if (wgt > 0) {
				return -getWeightedValueForPositives ((short)(-val), wgt);
			}
			else {
				return getWeightedValueForPositives ((short)(-val), (short)-wgt);
			}
		}
	} 

	int getWeightedValueForPositives (short val, short wgt) {
		Integer i;
		Vector wgts;

		wgts = (Vector)vals.elementAt (val);
		i = (Integer)wgts.elementAt (wgt);
		return i.intValue ();
	} 

	void writeToStream (PrintWriter str) {
		short val, wgt, ival, iwgt, maxVal, maxWgt;
		int result;
		Vector wgts;
		
		str.println ("val;wgt;result");
		maxVal = (short)vals.size ();
		wgts = (Vector)vals.elementAt (0);
		maxWgt = (short)wgts.size ();
		for (ival=(short)(-maxVal+1); ival<maxVal; ival++) {
			for (iwgt=(short)(-maxWgt+1); iwgt<maxWgt; iwgt++) {
				result = getWeightedValue (ival, iwgt);
				str.println (ival+";"+iwgt+";"+result);
			}
		}
	}	
}
